@@ -1382,4 +1382,8 @@ def subscribe(channel):
 
 # Unsubscribe this session from receiving events from the named channel
 def unsubscribe(channel):
-    return anvil.server.call("anvil.private.unsubscribe", channel)
\ No newline at end of file
+    return anvil.server.call("anvil.private.unsubscribe", channel)
+
+
+def get_subscriptions():
+    return anvil.server.call("anvil.private.get_subscriptions")
\ No newline at end of file
