@@ -42,7 +42,8 @@ from ._server import (register,
                       get_client_session,
                       get_session_id,
                       subscribe,
-                      unsubscribe)
+                      unsubscribe,
+                      get_subscriptions)
 
 _threaded_server.send_reqresp = lambda r, collect_capabilities=None: _get_connection().send_reqresp(r, collect_capabilities=collect_capabilities)
 
